import _find from 'lodash/find';
import React from 'react';
import PropTypes from 'prop-types';
import ImmutablePropTypes from 'react-immutable-proptypes';
import { Map as Map$1, List, fromJS } from 'immutable';
import Select from 'react-select';
import { colors, WidgetPreviewContainer } from '@igloo-be/netlify-cms-ui-default';

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _objectSpread(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};
    var ownKeys = Object.keys(source);

    if (typeof Object.getOwnPropertySymbols === 'function') {
      ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) {
        return Object.getOwnPropertyDescriptor(source, sym).enumerable;
      }));
    }

    ownKeys.forEach(function (key) {
      _defineProperty(target, key, source[key]);
    });
  }

  return target;
}

const styles = {
  control: styles => _objectSpread({}, styles, {
    border: 0,
    boxShadow: 'none',
    padding: '9px 0 9px 12px'
  }),
  option: (styles, state) => _objectSpread({}, styles, {
    backgroundColor: state.isSelected ? `${colors.active}` : state.isFocused ? `${colors.activeBackground}` : 'transparent',
    paddingLeft: '22px'
  }),
  menu: styles => _objectSpread({}, styles, {
    right: 0,
    zIndex: 2
  }),
  container: styles => _objectSpread({}, styles, {
    padding: '0 !important'
  }),
  indicatorSeparator: (styles, state) => state.hasValue && state.selectProps.isClearable ? _objectSpread({}, styles, {
    backgroundColor: `${colors.textFieldBorder}`
  }) : {
    display: 'none'
  },
  dropdownIndicator: styles => _objectSpread({}, styles, {
    color: `${colors.controlLabel}`
  }),
  clearIndicator: styles => _objectSpread({}, styles, {
    color: `${colors.controlLabel}`
  }),
  multiValue: styles => _objectSpread({}, styles, {
    backgroundColor: colors.background
  }),
  multiValueLabel: styles => _objectSpread({}, styles, {
    color: colors.textLead,
    fontWeight: 500
  }),
  multiValueRemove: styles => _objectSpread({}, styles, {
    color: colors.controlLabel,
    ':hover': {
      color: colors.errorText,
      backgroundColor: colors.errorBackground
    }
  })
};

function optionToString(option) {
  return option && option.value ? option.value : '';
}

function convertToOption(raw) {
  if (typeof raw === 'string') {
    return {
      label: raw,
      value: raw
    };
  }

  return Map$1.isMap(raw) ? raw.toJS() : raw;
}

class SelectControl extends React.Component {
  constructor(...args) {
    super(...args);

    _defineProperty(this, "handleChange", selectedOption => {
      const onChange = this.props.onChange;

      if (Array.isArray(selectedOption)) {
        onChange(fromJS(selectedOption.map(optionToString)));
      } else {
        onChange(optionToString(selectedOption));
      }
    });

    _defineProperty(this, "getSelectedValue", ({
      value,
      options,
      isMultiple
    }) => {
      if (isMultiple) {
        const selectedOptions = List.isList(value) ? value.toJS() : value;

        if (!selectedOptions || !Array.isArray(selectedOptions)) {
          return null;
        }

        return selectedOptions.filter(i => options.find(o => o.value === (i.value || i))).map(convertToOption);
      } else {
        return _find(options, ['value', value]) || null;
      }
    });
  }

  render() {
    const _this$props = this.props,
          field = _this$props.field,
          value = _this$props.value,
          forID = _this$props.forID,
          classNameWrapper = _this$props.classNameWrapper,
          setActiveStyle = _this$props.setActiveStyle,
          setInactiveStyle = _this$props.setInactiveStyle;
    const fieldOptions = field.get('options');
    const isMultiple = field.get('multiple', false);
    const isClearable = !field.get('required', true) || isMultiple;

    if (!fieldOptions) {
      return React.createElement("div", null, "Error rendering select control for ", field.get('name'), ": No options");
    }

    const options = [...fieldOptions.map(convertToOption)];
    const selectedValue = this.getSelectedValue({
      options,
      value,
      isMultiple
    });
    return React.createElement(Select, {
      inputId: forID,
      value: selectedValue,
      onChange: this.handleChange,
      className: classNameWrapper,
      onFocus: setActiveStyle,
      onBlur: setInactiveStyle,
      options: options,
      styles: styles,
      isMulti: isMultiple,
      isClearable: isClearable,
      placeholder: ""
    });
  }

}

_defineProperty(SelectControl, "propTypes", {
  onChange: PropTypes.func.isRequired,
  value: PropTypes.node,
  forID: PropTypes.string.isRequired,
  classNameWrapper: PropTypes.string.isRequired,
  setActiveStyle: PropTypes.func.isRequired,
  setInactiveStyle: PropTypes.func.isRequired,
  field: ImmutablePropTypes.contains({
    options: ImmutablePropTypes.listOf(PropTypes.oneOfType([PropTypes.string, ImmutablePropTypes.contains({
      label: PropTypes.string.isRequired,
      value: PropTypes.string.isRequired
    })])).isRequired
  })
});

const SelectPreview = ({
  value
}) => React.createElement(WidgetPreviewContainer, null, value ? value.toString() : null);

SelectPreview.propTypes = {
  value: PropTypes.string
};

export { SelectControl, SelectPreview };
